#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl

BEGIN {
  require './pmx-viruscheck-lib.pl';
}


use HTML::Template;
use ActiveState::Config qw(:path);;
use PureMessage::Manager::HTMLHelper;
use HTTP::Date qw/time2isoz/;

use strict;

our %text;
our %in;
our %config;
our @errors;
my  %vars;
 
eval {

  my $HEADER_REGEX = qr/^([a-zA-Z0-9\-]+):\s*(.*)$/;
  my @VALID_MACROS = qw( SUBJECT HOSTNAME VIRUS_IDS QID ADMIN_ADDR);
  my %VALID_MACROS = map { ($_ => 1) } qw (SUBJECT HOSTNAME VIRUS_IDS QID ADMIN_ADDR);
  
  my $action = $in{'action'};


  if (!$action) {
    foreach my $key ( qw( save  )) {
      $action = $key if exists $in{"${key}.x"};
      last if $action;
    }
  }



  $vars{SUBJECT} = $in{'subject'};
  $vars{BODY} = $in{'body'};
  $vars{HEADERS} = removeExtraneousEmptyLines($in{'headers'});
  
  if ($action eq 'save') {

      if ( validEmail() && writeTmplFile()) {
         webmin_log('modified','email template','viruscheck',\%in);
         &header(&text('email_title'), "", undef, 0, 1);
         $vars{BODY} = n2br($vars{BODY});
         $vars{HEADERS} = n2br($vars{HEADERS});
         putTemplateUpdated();
      } else {
        &header(&text('email_title'), "", undef, 0, 1);
        putEmail(0);
      }
      &footer();
     
  } elsif ($action eq 'state') {

      validEmail();
      &header(&text('email_title'), "", undef, 0, 1);
      putEmail(0);
      &footer();
     
  } else {
    &header(&text('email_title'), "", undef, 0, 1);
    putEmail(1);
    &footer();
  }

  unlock_all_files();

### start of subs

sub removeExtraneousEmptyLines {
  my $intext = shift;
  my $line;
  foreach my $header (split(/\n/,$intext)) {
    next if $header =~ /^\s*$/;
    $line .= "$header\n";
  }
  return $line;
}

sub validEmail {

  # make sure header lines are valid
  foreach my $header (split(/\n/,$vars{HEADERS})) {
    next if $header =~ /^\s*$/;
    raise_error('error_invalid_header_line',$header) unless $header =~ $HEADER_REGEX;
  }
  
  while ($vars{HEADERS} =~ /%%([^%]+)%%/g) {
    my $a = $1;
    raise_error('error_invalid_header_macro',$a) unless $VALID_MACROS{$a};
  }
  
  while ($vars{BODY} =~ /%%([^%]+)%%/g) {
    my $a = $1;
    raise_error('error_invalid_macro',$a) unless $VALID_MACROS{$a};
  } 
  
  raise_error('error_subject_empty') unless $vars{SUBJECT};
  raise_error('error_body_empty') unless $vars{BODY};
  return 1 unless $#errors != -1;
  return;
}

sub putTemplateUpdated {

  theme_dialog(&text('email_updated_section_header'),
             $text{'email_updated_dialog_text'},"-information",undef,undef,["email_template.cgi",'-ok','Continue']);

 
}


sub putEmail {
  my $load = shift;
 
  $vars{HEADER} = theme_section_header(&text('email_section_header'));  
 
   my  $template = HTML::Template->new(filename => 'tmpl/email_template.html',
                                       die_on_bad_params => 0,
                                     );
 
  $vars{FOOTER} = theme_section_footer(['save','-save','Save','save'],
                                       ['..','-cancel','Cancel'],
                                      );
                                      
    
  $vars{TEMPLATE_FILE} = getTmplFileName();

  # attempt to load the template if we have to

  if ($load) {
    getTmplFile();
  }
  
  $template->param(\%vars);
  print $template->output;

}

# headers: 
# Subject: Virus quarantined by PureMessage
# Content-Type: text/plain
# X-PureMessage-Wrap: 60

sub getTmplFileName {
  return $config{filter}->{'PureMessage::VirusCheck'}->{virus_template};
}

sub backUpFile{
  my $file = shift;
  return unless $file;

  # generate a new backup filename
  my $newfilename = $file;
  $newfilename = "$file.bak." . time2isoz() while(-e $newfilename);
 
  open FVIN, "$file" || return; 
  open FVOUT, ">$newfilename" || return;
  
  print FVOUT $_ while(<FVIN>);
  
  close FVOUT;
  close FVIN;
  return 1;
}
  

sub writeTmplFile {
  my $filterfile = getTmplFileName();
  lock_file($filterfile);
  if (-e $filterfile and -w _ ) {
    raise_error('error_email_unable_to_backup',$filterfile) && return unless backUpFile($filterfile);

    if (open(FV,">$filterfile")) {
      print FV "Subject: $vars{SUBJECT}\n";
      print FV "$vars{HEADERS}\n";
      print FV $vars{BODY};
      close FV;
    } else {
      raise_error('error_email_unable_to_write_file',$!);
    }

  } else {
      raise_error('error_email_unable_to_open_file',$filterfile);
  }
  unlock_file($filterfile);
  return 1 if $#errors == -1;
  return;
}

sub getTmplFile {
  # template file is in conf
  my $lines;
  my $headers;
  my $filterfile = getTmplFileName();

  if (-e $filterfile and -r _) {
    if (open(FV,$filterfile)) {
      # parse off the headers, 
      # if it's not one we're expecting then put it into $headers and we'll show
      # them a custom block of headers
      my $in_headers = 1;
      while(my $inline = <FV>) {
        if ($in_headers) {
        
          if ($inline =~ $HEADER_REGEX) {
            # then it's a header..
            my ($name,$value) = ($1,$2);
            if ($name =~ /Subject/i) {
              $vars{SUBJECT} = $value;
            } else {
              $headers .= $inline;
            }
          } else {
            # it's not a header, dump it into the body and jump out of headers
            $in_headers = 0;
            $lines .= $inline unless $inline =~ /^\s*$/; # but don't put the eoh \n
          }
          
        } else {
          $lines .= $inline;
        }
      }
      close FV;
      $vars{BODY} = $lines;
      $vars{HEADERS} = $headers;
    } else {
      raise_error('error_email_error_reading_file',$!);
    }
  } else {
    raise_error('error_email_unable_to_read_file');
  }
}

};

&header(&text('email_title'), "", undef, 0, 1) if $@;
print "ERROR: $@" if $@;

