#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl -w
use strict;
use warnings;
use PureMessage::Service;
use PureMessage::Manager::Request qw($REQUEST %in);
use PureMessage::Manager::Lang qw(&text);
use PureMessage::Manager::Response::Services;
use PureMessage::Policy::Virus::Config qw(%config);
use PureMessage::Manager::Widget;
use ActiveState::Config qw(:path);
use PureMessage::Service;

my $response = $REQUEST->get_response(
    'PureMessage::Manager::Response::Services',
    stderr_to_stdout => 1);

# create a generic widget object so that we can extract the "save" toolbar 
# button value if clicked.
my $W = PureMessage::Manager::Widget->new();
$W->read_input($REQUEST);
my $tb_data = $W->result_data();

$response->add_help(text => "Help", tab => "locserv", page => "mgr_locserv_vscan_edit");
$response->title(text('vscan_edit_title'));

my %form_data = %{$config{daemon}};
if ($tb_data->{'save'}) {

    # Save the config
    my $cfg = PureMessage::Policy::Virus::Config->new(writable => 1);
    eval { _save($cfg, %in) };
    if ($@) {
	$response->raise_error(text("vscan_save_error", $@));
	%form_data = %in;
    }
    else {
	my $running = _is_vscan_running();
	$response->dialog(
	    style => 'DLG_INFO',
	    dialog_title => text('vscan_save_ok_title'),
	    msg => $running ? text('vscan_save_ok_restart_msg') :
		              text('vscan_save_ok_start_msg'),
	    buttons => [[
		{type => 'text', 'text' => text('ok'), href=>'index.cgi'}
		]],
		);
	$response->send();
	$response->log('Update VScan', '', 'VScan', \%in);
	exit;
    }
}

# show the config data
$response->add_part(build_ui(%form_data), 'body_form');
$response->send;

sub build_ui {
    my %data = @_;

    my @buttons;
    push(@buttons, {
	    type => 'icon', 
	    icon => 'save',
	    name => 'save',
	    value => 'save',
	    text => text('save'),
	    alt  => text('save'),
	});

    push(@buttons, {
	    type => 'icon', 
	    icon => 'cancel',
	    href => 'index.cgi', 
	    text => text('cancel'),
	    alt  => text('cancel'),
	});

    my %state;
    my $tb = $response->ToolBar(
	buttons => [\@buttons], 
	state   => \%state,
	name    => 'tb',
	width   => 600,
    );
    
    # The Form widget's data keys and form labels must match
    my $fields = [
	text('vscan_lbl_concurrency_limit'),
	text('vscan_lbl_reinit_limit'),
	text('vscan_lbl_inactive_limit'),
	text('vscan_lbl_queue_limit'),
	text('vscan_lbl_scan_limit'),
	text('vscan_lbl_respawn_wait'),
	text('vscan_lbl_port'),
    ];

    my $formdata;
    foreach (qw(concurrency_limit reinit_limit inactive_limit
	        queue_limit scan_limit respawn_wait port)) {
	$formdata->{text("vscan_lbl_$_")} = _txt($_, $data{$_});
    }

    my $f = $response->Form(
	name    => 'edit_page',
	data    => $formdata,
	fields  => $fields, 
	toolbar => $tb,
	columns => 2,
	title   => text('vscan_service_edit'),
	action  => 'edit.cgi',
	width   => 600,
	);
    return $f;
}


sub _txt {
    my ($name, $val) = @_;
    $val = '' unless $val;
    my $str = '<input type="text" class="stretch" name="';
    $str .= $name . '" value="' . $val . '">';
    return $str;
}

sub _blank() { '&nbsp;' }


sub _save {
    my ($cfg, %data) = @_;
    my $daemon = path_node($cfg->root, "daemon");
    foreach (qw(port concurrency_limit reinit_limit inactive_limit
	        queue_limit scan_limit respawn_wait)) {
	if (defined $data{$_} && path_value($daemon, $_) ne $data{$_}) {
	    path_value($daemon, $_, $data{$_});
	}
    }
    $cfg->save;
}

sub _is_vscan_running {
    foreach (PureMessage::Service->Available()) {
	return 1 if $_->type eq "vscan" && $_->running;
    }
    return 0;
}

