#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl -w
use strict;
use warnings;
use PureMessage::Service;
use PureMessage::Manager::Request qw($REQUEST %in);
use PureMessage::Manager::Lang qw(&text);
use PureMessage::Manager::Response::Services;
my $response = $REQUEST->get_response(
    'PureMessage::Manager::Response::Services',
    stderr_to_stdout => 1);

$response->add_help(text => "Help", tab => "locserv", page => "mgr_locserv_vscan");
$response->title(text('vscan_index_title'));

my $vscan;
foreach (PureMessage::Service->Available()) {
    $vscan = $_ if $_->type eq "vscan";
}

unless ($vscan) {
    $response->print_notification(text('vscan_not_available'));
    $response->send;
    exit;
}

eval {
    if ($REQUEST->query('restart')) {
	$vscan->restart;
	$response->print_notification(text('vscan_restarted'));
	sleep 1;
    }

    if ($REQUEST->query('start')) {
	$vscan->start;
	$response->print_notification(text('vscan_started'));
	sleep 1;
    }

    if ($REQUEST->query('stop')) {
	$vscan->stop;
	$response->print_notification(text('vscan_stopped'));
	sleep 1;
    }
};
if ($@) {
    if ($@ =~ /^Start failed: no backends/) {
	$response->raise_error(text("vscan_no_backends"));
    }
    else {
	$response->raise_error($@);
    }
}

my $status = $vscan->status;
$status->{status} = $status->{running} ? text('status_running') : 
    text('status_stopped');
    
my @buttons;
push(@buttons, {
	type => 'icon', 
	icon => 'start',
	href => 'index.cgi?start=1',
	text => text('start'),
	alt => text('start'),
    }) unless $vscan->running;
    
push(@buttons, {
	type => 'icon', 
	icon => 'restart',
	href => 'index.cgi?restart=1',
	text => text('restart'),
	alt => text('restart'),
    }) if $vscan->running;

push(@buttons, {
	type => 'icon', 
	icon => 'stop',
	href => 'index.cgi?stop=1',
	text => text('stop'),
	alt => text('stop'),
    }) if $vscan->running;

push(@buttons, {
	type => 'icon', 
	icon => 'edit',
	href => 'edit.cgi', 
	text => text('edit'),
	alt => text('edit'),
    });

my %state;
my $tb = $response->ToolBar(
    buttons => [\@buttons], 
    state   => \%state,
    name    => 'tb',
);

# The Form widget's data keys and form labels must match
my $fields = [
    text('vscan_lbl_status'),
    text('vscan_lbl_pid'),
    text('vscan_lbl_port'),
    text('vscan_lbl_connections'),
    text('vscan_lbl_queued'),
    text('vscan_lbl_numscanners'),
];

my $formdata = {
    text('vscan_lbl_status')      => $status->{status},
    text('vscan_lbl_pid')         => $status->{pid},
    text('vscan_lbl_port')        => $status->{port},
    text('vscan_lbl_connections') => $status->{connections},
    text('vscan_lbl_queued')      => $status->{queued},
    text('vscan_lbl_numscanners') => $status->{numScanners},
};

my $f = $response->Form(
    name    => 'status_page',
    data    => $formdata,
    fields  => $fields, 
    toolbar => $tb,
    columns => 2,
    title   => text('vscan_service_status'),
    action  => 'index.cgi',
    );

$response->add_part($f, 'body_form');
$response->send;

