package PureMessage::Policy::ArchiveInfo;
use strict;
use base 'Exporter';
our @EXPORT_OK = qw(archive_get_filenames);

sub archive_get_filenames {
    # for now we only support zip
    # maybe this will be extended someday
    return _zip_get_filenames(@_);
}

{
    # PKZIP file format miscellaneous constants

    use constant SIGNATURE_FORMAT => "V";
    use constant SIGNATURE_LENGTH => 4;

    use constant LOCAL_FILE_HEADER_SIGNATURE => 0x04034b50;
    use constant LOCAL_FILE_HEADER_FORMAT    => "v3 V4 v2";
    use constant LOCAL_FILE_HEADER_LENGTH    => 26;

    use constant SPLIT_ARCHIVE_SIGNATURE => 0x08074b50;
    use constant SPANNING_MARKER_SIGNATURE => 0x30304b50;
    
    use constant DATA_DESCRIPTOR_FORMAT => "V3";
    use constant DATA_DESCRIPTOR_LENGTH => 12;

    use constant GPBF_HAS_DATA_DESCRIPTOR_MASK   => 1 << 3;

    # best effort filename extraction.  
    # if something weird happens, just jump out
    # and return whatever filenames were found.
    sub _zip_get_filenames {
        my $fh = shift;
        my @filenames;
        seek $fh, 0, 0;
        while (1) {
            my $sig = _next_sig($fh);
            last unless $sig;
            # possible split archive sig here
            if ($sig == SPLIT_ARCHIVE_SIGNATURE or $sig == SPANNING_MARKER_SIGNATURE) {
                $sig = _next_sig($fh);
            }
            last unless $sig == LOCAL_FILE_HEADER_SIGNATURE;
            my $read = read $fh, my($lfh), LOCAL_FILE_HEADER_LENGTH;
            last unless $read == LOCAL_FILE_HEADER_LENGTH;
            my (undef, 
                $gp_flag, 
                undef, 
                undef, 
                undef, 
                $compressed_size,
                undef,
                $filename_len,
                $extra_field_len,
                ) = unpack(LOCAL_FILE_HEADER_FORMAT, $lfh);   
            last unless $filename_len > 0;
            $read = read $fh, my($filename), $filename_len;
            last unless $read == $filename_len;
            push @filenames, $filename;
            last unless seek $fh, $extra_field_len + $compressed_size, 1;
            my $seek = $gp_flag & GPBF_HAS_DATA_DESCRIPTOR_MASK ? DATA_DESCRIPTOR_LENGTH : 0;
            # might be a split archive signature next
            $sig = _next_sig($fh);
            unless ($sig == SPLIT_ARCHIVE_SIGNATURE or $sig == SPANNING_MARKER_SIGNATURE) {
                # it wasn't a sig after all
                $seek -= 4;
            }
            if ($seek) {
                last unless seek $fh, $seek, 1;
            }
        }
        # dont return dirs, only files
        return grep m#[^/]$#, @filenames;
    }
    sub _next_sig {
        my $fh = shift;
        my $read = read($fh, my $buf, SIGNATURE_LENGTH);
        return 0 unless $read == SIGNATURE_LENGTH;
        return unpack(SIGNATURE_FORMAT, $buf);
    }
}

1;
