package PureMessage::Policy::Virus::Client;

use strict;
use PureMessage::Socket;
use ActiveState::Milter::SockSpec;
use PureMessage::Policy::Virus::Config '%config';
use PureMessage::Policy::Virus::Protocol qw(send recv);

sub new {
    my $cls = shift;
    my @sock = ActiveState::Milter::SockSpec::parse($config{daemon}{port});
    my $self = bless {
	port	    => $config{daemon}{port},
	sockspec    => \@sock,
	buf	    => '',
    }, ref($cls) || $cls;
}

sub connect {
    my $self = shift;
    my ($scheme, $port, $if) = @{$self->{sockspec}};
    if ($scheme eq 'inet') {
	$if ||= '127.0.0.1';
	$self->{sock} = PureMessage::Socket::INET->new(
	    PeerAddr	=> $if,
	    PeerPort	=> $port,
	    Type	=> SOCK_STREAM,
	);
    }
    elsif ($scheme eq 'unix') {
	$self->{sock} = PureMessage::Socket::UNIX->new(
	    Peer	=> $port,
	    Type	=> SOCK_STREAM,
	);
    }
    else {
	die "unsupported socket scheme $scheme";
    }
    die "can't connect to $self->{port}: $!" unless defined $self->{sock};
}

sub disconnect {
    my $self = shift;
    my $sock = $self->{sock} or return;
    $sock->close if $sock->connected;
}

sub scan_file_async {
    my $self = shift;
    for my $file (@_) {
	send($self, 's', file => $file);
    }
}

sub scan_file_report {
    my $self = shift;
    my $count = 1;
    my $total;
    my %rept;
    while ($count--) {
	my ($op, @kv) = recv($self) or last;
	if ($op eq 's') {
	    my ($file, $backend, @v);
	    while (@kv) {
		my ($k, $v) = splice(@kv, 0, 2);
		if ($k eq 'file') {
		    $file = $v;
		}
		elsif ($k eq 'backend') {
		    $backend = $v;
		}
		elsif ($k eq 'backends' and not $total) {
		    $total = $v * @_;
		    $count = $total - 1;
		}
		elsif ($k eq 'v') {
		    push @v, $v;
		}
	    }
	    $rept{$file}{$backend} = \@v;
	}
	elsif ($op eq 'c') {
	    my %kv = @kv;
	    unless ($total) {
		$total = $kv{backends} * @_;
		$count = $total - 1;
	    }
	    $rept{$kv{file}}{$kv{backend}} = $kv{status};
	}
	elsif ($op eq 'e') {
	    my %err = @kv;
	    my $file = delete $err{file};
	    $rept{$file} = \%err;
	}
	else {
	    die "invalid opcode '$op' from vscan daemon";
	}
    }
    %rept;
}

sub clean_file_async {
    my $self = shift;
    for my $file (@_) {
	send($self, 'c', file => $file);
    }
}

sub scan_file {
    my $self = shift;
    my $file = shift;
    send($self, 's', file => $file);
    my ($op, @kv) = recv($self);
}

sub status {
    my $self = shift;
    send($self, 'p');
    my ($op, @kv) = recv($self);
}

sub softstop {
    my $self = shift;
    send($self, 'x', hard => 0);
    my ($op, @kv) = recv($self);
}

sub hardstop {
    my $self = shift;
    send($self, 'x', hard => 1);
    # Don't wait for a reply -- just quit
}

sub softrespawn {
    my $self = shift;
    send($self, 'r', hard => 0);
    my ($op, @kv) = recv($self);
}

sub hardrespawn {
    my $self = shift;
    send($self, 'r', hard => 1);
    # Don't wait for a reply -- just quit
}

sub restart {
    my $self = shift;
    send($self, 'R');
    my ($op, @kv) = recv($self);
}

1;
