package PureMessage::Policy::Virus::Protocol;

use strict;
use ActiveState::ProcessPool qw(encode_kv decode_kv);

use base 'Exporter';

our @EXPORT_OK = qw(send recv);

# Send a packet to the remote side
sub send {
    my $sock = shift->{sock};
    my $op   = shift;
    my $data = encode_kv(@_);
    my $pack = pack('aN/a*', $op, $data);
    my $len = length $pack;
    my $written = 0;
    while ($written < $len) {
	my $left   = $len - $written;
	my $nw     = $sock->syswrite($pack, $left, $written)
	    or die "short write trying to write $left bytes: $!";
	$written += $nw;
    }
    $len;
}

sub HEADER() {   5}
sub CHUNK()  {4096}

# Read one or more packets from the remote side. Only a single packet is
# returned; the rest of the packets are saved in a buffer for future decoding.
sub recv {
    my $obj = shift;
    if (not defined $obj->{buf} or length $obj->{buf} < HEADER) {
	readchunk($obj) or return;
    }
    my ($op, $datalen, $data) = unpack('aN', $obj->{buf});
    my $size = $datalen + HEADER;
    while (length $obj->{buf} < $size) {
	readchunk($obj) or return;
    }
    $data = substr($obj->{buf}, HEADER, $datalen);
    die "len(data)=" . length($data) . " != datalen=$datalen"
    	unless length $data == $datalen;
    my @kv = decode_kv($data);
    substr($obj->{buf}, 0, $size, '');
    return $op, @kv;
}

sub readchunk {
    my $obj = shift;
    my $buf;
    my $nread = $obj->{sock}->sysread($buf, CHUNK)
	or return;
    $obj->{buf} = '' unless defined $obj->{buf};
    $obj->{buf} .= $buf;
    $nread;
}

1;
