package PureMessage::Service::vscan;
use warnings;
use strict;
use base 'PureMessage::Service';
use PureMessage::Config qw(expand_filename);
use ActiveState::Handy ();
use PureMessage::Policy::Virus::Client;
use PureMessage::Policy::Virus::Config;
use ActiveState::Config::Depend qw(config_mtime_deps);
use Carp;

my $vscan_d = expand_filename("etc/init.d/pmx-vscan");
my $vscan_port = $PureMessage::Policy::Virus::Config::config{daemon}->{port};

# manual imports so we don't pollute the method name space
*_run = \&ActiveState::Handy::run;

sub Available {
    my $class = shift;
    return ($class->new);
}

sub type { "vscan" }

sub name { "vscan" }

sub start {
    my ($self, %opts) = @_;
    my @cmd = ($vscan_d, "start");
    my $rc = _exec(\@cmd, %opts);
    croak "Start failed: no backends configured to use the vscan daemon."
	if ($? >> 8) == 3;
    return $rc;
}

sub stop {
    my ($self, %opts) = @_;
    my @cmd = ($vscan_d, "stop");
    _exec(\@cmd, %opts);
}

sub restart {
    my ($self, %opts) = @_;
    my @cmd = ($vscan_d, "restart");
    my $rc = _exec(\@cmd, %opts);
    croak "Start failed: no backends configured to use the vscan daemon."
	if ($? >> 8) == 3;
    return $rc;
}

sub status {
    my $c = PureMessage::Policy::Virus::Client->new();
    my %status;
    eval { $c->connect };
    if ($@) {
        $status{ErrorMsg} = "Failed to connect to the AntiVirus Service. $@";
        $status{running} = 0;
	return \%status;
    }

    # XXX should document what values can be expected from a status ping.
    my $op;
    ($op, %status) = $c->status;
    $status{port} = $vscan_port;
    if ($op) {
	# $status already contains pid.
	$status{running} = 1;
	my $s = '';
	$s .= $status{numScanners};
	$s .= $status{numScanners} == 1 ?  " scanner, " : " scanners, ";
	$s .= $status{connections};
	$s .= $status{connections} == 1 ? " connection, " : " connections, ";
	$s .= $status{queued} . " queued.";
	$status{summary} = $s;
	my $mtime = config_mtime_deps(expand_filename('etc/virus.conf'));
	$status{config} = (($status{start_time} || 0) < $mtime) ? 
	    "stale" : "sync";
    }
    else {
        $status{running} = 0;
	$status{ErrorMsg} = $@ if $@;
    }
    return \%status;

}

sub running {
    my $self = shift;
    return $self->status->{running};
}

sub _exec {
    my $cmd = shift;
    my %opts = @_;

    if ($opts{bg}) {
	if (my $pid = _bg()) {
	    return $pid;
	}	    
	exec(@$cmd);
	die "Can't exec: $!";
    }
    $$cmd[0] = '@-' . $$cmd[0];
    _run(@$cmd);
}

# XXX This should be exported by the base class as each service will
# want to use this.
sub _bg {
    my $pid = fork();
    die "Can't fork: $!" unless defined $pid;
    return $pid if $pid;
    
    open(STDIN, "</dev/null");
    open(STDOUT, ">/dev/null");
    open(STDERR, ">/dev/null");
    return 0;
}

1;
